class ApiFeatures {
  constructor(query, queryString) {
    this.query = query;
    this.queryString = queryString;
  }

  filter() {
    const queryObj = { ...this.queryString };
    const excludedFields = ['sort', 'fields', 'page', 'limit'];

    excludedFields.forEach((el) => delete queryObj[el]);

    let queryString = JSON.stringify(queryObj);
    queryString = queryString.replace(
      /\b(gt|gte|lt|lte\b)/g,
      (match) => `$${match}`
    );

    this.query = this.query.find(JSON.parse(queryString));

    return this;
  }

  sort(defaultSort) {
    if (this.queryString.sort) {
      const sortBy = this.queryString.sort.split(',').join(' ');
      this.query = this.query.sort(sortBy);
    } else {
      this.query = this.query.sort(defaultSort);
    }

    return this;
  }

  limitFields(...defaultExcludedFields) {
    let fields;

    defaultExcludedFields.forEach((el, i) => {
      if (!el.startsWith('-')) {
        defaultExcludedFields[i] = `-${el}`;
      }
    });

    if (this.queryString.fields) {
      fields = this.queryString.fields.split(',').join(' ');
    } else {
      if (defaultExcludedFields.length === 0) {
        fields = '-__v';
      } else {
        fields = defaultExcludedFields.join(' ');
      }
    }

    this.query = this.query.select(fields);

    return this;
  }

  paginate(defaultPage = 1, defaultLimit = 100) {
    const page = this.queryString.page * 1 || defaultPage;
    const limit = this.queryString.limit * 1 || defaultLimit;
    const skip = (page - 1) * limit;

    this.query = this.query.skip(skip).limit(limit);

    return this;
  }
}

module.exports = ApiFeatures;
